!*************************************************************************************************
! output file maker (for point simulation)
!*************************************************************************************************
SUBROUTINE output_point &
   (year, dom, doy, counter, &
   tmp_air, tmp_soil, cloud, prec, humid, wind, &
   W_fi, W_wilt, W_sat, W_mat, &
   co2atm, ap, vp, vpd, dnsa, vp_sat, slope_vps, &
   sl_hgt, dlen, rad, par, par_grass, &
   albedo_soil, albedo_veg, radnet_soil, radnet_veg, &
   mort_regu3, gdd, moist_factor, fire_factor, &
   pool_w1, pool_w2, pool_snow, canopy_cond, &
   pool_litter, pool_som_int, pool_som_slow, &
   stat_water, eK, &
   pft, age, height, bole, dbh_heartwood, dbh_sapwood, crown_diameter, crown_area, la, &
   mass_leaf, mass_trunk, mass_root, mass_stock, mass_available, &
   bole_x, bole_y, crown_x, crown_y, tree_exist, height_limit, radius_limit,&
   type_grass, gmass_leaf, gmass_root, gmass_available, gmass_stock, &
   lai_max_wood, lai_max_grass, lai_mean_wood, lai_mean_grass, lai_grass, lai_opt_grass, &
   pft_exist, biome, C3g_no, C4g_no, &
   phenology, pheno_clo, pheno_cls, pheno_gdd, pheno_ncd, &
   c_emission, c_uptake, gpp, npp, gpp_annual, npp_annual, mort_regu1, &
   hydRecord_preci, hydRecord_runoff, hydRecord_ic, hydRecord_ev, hydRecord_tr)

USE data_structure
implicit none

!loop counters
integer,intent(IN)::year     !loop counter for year (1-Simulation_year)
integer,intent(IN)::dom      !loop counter for day of month (1-31)
integer,intent(IN)::doy      !day of the year (1-365)
integer,intent(IN)::counter  !simulation day (1-Simulation_year*365)

!Climate data
real,intent(IN)::tmp_air     !2m air temperature (Celcius)
real,intent(IN)::tmp_soil    !soil temperature (Celcius)
real,intent(IN)::cloud       !total cloudiness (fraction)
real,intent(IN)::prec        !precipitation (mm day-1)
real,intent(IN)::humid       !specific humidity (kg kg-1)
real,intent(IN)::wind        !wind  velocity (m s-1)

!Location data
real,intent(IN)::W_fi         !filed capacity   (m3/m3, 0.0 -> 1.0)
real,intent(IN)::W_wilt       !wilting point    (m3/m3, 0.0 -> 1.0)
real,intent(IN)::W_sat        !saturate point   (m3/m3, 0.0 -> 1.0)
real,intent(IN)::W_mat        !matrix potential (m, -0.0001 -> -3.0)

!air related
real,intent(IN)::co2atm    !ambient (canopy) CO2 concentration (ppmv)
real,intent(IN)::ap        !air pressure (hPa)
real,intent(IN)::vp        !vapour pressure (hPa)
real,intent(IN)::vpd       !vapour pressure deficit (hPa)
real,intent(IN)::dnsa      !density of air (kg m-3)
real,intent(IN)::vp_sat    !saturation vapour pressure (hPa)
real,intent(IN)::slope_vps !slope of saturation vapour pressure related to temp (hPa deg C-1)

!radiation related
real,intent(IN)::sl_hgt       !solar hight at midday (degree)
real,intent(IN)::dlen         !day length (hour)
real,intent(IN)::rad          !global radiation at the ground surface (W/m2)
real,intent(IN)::par          !par of mid-day (micro mol photon m-2 s-1)
real,intent(IN)::par_grass    !par on top of the grass layer (micro mol photon m-2 s-1)

real,intent(IN)::albedo_soil  !soil surface albedo
real,intent(IN)::albedo_veg   !vegetation area albedo
real,intent(IN)::radnet_soil  !soil surface net radiation, short wave (W m-2)
real,intent(IN)::radnet_veg   !canopy net radiation, short wave (W m-2)

!climate related recorder
real,intent(IN)::mort_regu3   !heat stress for each year for mortality calc.
real,intent(IN)::gdd          !growing-degree-day-sum of 5 Celcius base
real,intent(IN)::moist_factor !adjusted moisture factor for fire spread
real,intent(IN)::fire_factor  !factor for fire probability calculation

!hydrogy related
real,intent(IN)::pool_w1      !soil water content of soil layer 1 (mm)
real,intent(IN)::pool_w2      !soil water content of soil layer 2 (mm)
real,intent(IN)::pool_snow    !water equivalent snow depth (mm)
real,intent(IN)::canopy_cond  !monthly stomatal conductance   (mol H2O m-2 s-1)

!soil carbon (s.o.m.; soil organic matter)
real,intent(IN)::pool_litter   !litter pool in a stand (g drymatter / stand)
real,intent(IN)::pool_som_int  !pool of s.o.m. with intermediated decomposition rate (g dm / stand)
real,intent(IN)::pool_som_slow !pool of s.o.m. with slow decomposition rate (g dm / stand)

!photosynthesis related
real,dimension(PFT_no)::stat_water   !state of water satisfactory (0.0-1.0)
real,dimension(PFT_no)::eK           !light attenuation coefficient, no dimension

!individual charactristics of woody PFT
integer,dimension(Max_no)::pft            !Plant Functional Type (1-PFT_no)
integer,dimension(Max_no)::age            !tree age (year)
integer,dimension(Max_no)::height         !tree height from 1.3m above ground (STEP)
integer,dimension(Max_no)::bole           !bole length from 1.3m above ground (STEP)
real   ,dimension(Max_no)::dbh_heartwood  !heartwood diameter at 1.3m   (m)
real   ,dimension(Max_no)::dbh_sapwood    !sapwood   diameter at 1.3m   (m)
real   ,dimension(Max_no)::crown_diameter !crown diameter (m)
real   ,dimension(Max_no)::crown_area     !crown area     (m2)
real   ,dimension(Max_no)::la             !leaf area      (m2)
real   ,dimension(Max_no)::mass_leaf      !biomass of foliage (g)
real   ,dimension(Max_no)::mass_trunk     !biomass of stem    (g)
real   ,dimension(Max_no)::mass_root      !biomass of root    (g)
real   ,dimension(Max_no)::mass_stock     !biomass for stock  (g)
real   ,dimension(Max_no)::mass_available !biomass available  (g)
real   ,dimension(Max_no)::bole_x         !x_location in forest stand (m)
real   ,dimension(Max_no)::bole_y         !y_location in forest stand (m)
real   ,dimension(Max_no)::crown_x        !x_location of crown (m)
real   ,dimension(Max_no)::crown_y        !y_location of crown (m)
logical,dimension(Max_no)::tree_exist     !tree exist flag
integer,dimension(Max_no)::height_limit   !maximum tree height that proximate trees permit
real   ,dimension(Max_no)::radius_limit   !maximum canopy radius that proximate trees permit

!grid charactristics of herbaceous PFT
integer type_grass           !grass type priority in this girid (3:C3 type, 4:C4 type)

real,dimension(2)::gmass_leaf      !c3/c4 herbaceous foliage   biomass in a unit area (g/m2)
real,dimension(2)::gmass_root      !c3/c4 herbaceous root      biomass in a unit area (g/m2)
real,dimension(2)::gmass_available !c3/c4 herbaceous available biomass in a unit area (g/m2)
real,dimension(2)::gmass_stock     !c3/c4 herbaceous stock     biomass in a unit area (g/m2)
real             ::lai_max_wood    !Wood  LAI @ annual maximum  (m2/m2)
real             ::lai_max_grass   !Grass LAI @ annual maximum  (m2/m2)
real             ::lai_mean_wood   !Wood  LAI @ annual maximum  (m2/m2)
real             ::lai_mean_grass  !Grass LAI @ annual maximum  (m2/m2)
real,dimension(2)::lai_grass       !c3/c4 leaf area index of grass (m2/m2)
real,dimension(2)::lai_opt_grass   !c3/c4 optimum LAI for grass (m2/m2)

!biome stat
logical,dimension(PFT_no)::pft_exist !flag: PFT exist or not (.TRUE. or .FALSE.)
integer                  ::biome     !biome type number (see subroutine biome_type for definition)

!allocated number of grass PFTs
integer C3g_no !PFT number of C3 grass
integer C4g_no !PFT number of C4 grass

!phenology related
logical,dimension(PFT_no)::phenology !Phenological status (T:foliation, F:dormance)
integer,dimension(PFT_no)::pheno_clo !counter number of last leaf onset
integer,dimension(PFT_no)::pheno_cls !counter number of last leaf shedding
real                     ::pheno_gdd !GDD on 5 degree base
                                     !(reset on 1 Jan for N-hemisphere, 1 July for S-hemisphere)
                                     !
integer                  ::pheno_ncd !Number of chilling day, defined as number of days on which
                                     !the daily mean air temperature is below 5 Cecius degree
                                     !(reset on 1 Nov for N-hemisphere, 1 May for S-hemisphere)
                                     !

!carbon flux recorder
real                  c_emission     !monthly carbon emission (C g / month stand)
real                  c_uptake       !monthly carbon uptake   (C g / month stand)
real,dimension(PFT_no)::gpp          !GPP for each pft (g dm / stand / day)
real,dimension(PFT_no)::npp          !NPP for each pft (g dm / stand / day)
real,dimension(PFT_no)::gpp_annual   !GPP for each pft (g dm / stand / year)
real,dimension(PFT_no)::npp_annual   !NPP for each pft (g dm / stand / year)
real,dimension(Max_no)::mort_regu1 !NPP annual (g dm / individual)

!hydrogy record
real hydRecord_preci  !sum of precipitation      (mm/month)
real hydRecord_runoff !sum of runoff water       (mm/month)
real hydRecord_ic     !sum of interception water (mm/month)
real hydRecord_ev     !sum of evaporated water   (mm/month)
real hydRecord_tr     !sum of tranpirated water  (mm/month)

!Daily output (on each specifyed year)
IF (year<=10 .or. year==20 .or. year==30 .or. year==40 .or. year==50 .or. year==100 .or. year==200) then
     
     Call output_climate &
          (year, doy, tmp_air, tmp_soil, cloud, prec, humid, wind, vp, vp_sat )
     
     Call output_air &
          (year, doy, ap, vp, vpd, dnsa, vp_sat, slope_vps)
     
     Call output_radiation &
          (year, doy, sl_hgt, dlen, rad, par, par_grass)
     
     Call output_netradiation &
          (year, doy, albedo_soil, albedo_veg, &
           radnet_soil, radnet_veg)
     
     Call output_water &
      (year, doy, pool_snow, pool_w1, pool_w2, W_fi, W_wilt, W_sat, W_mat)
     
     Call output_grass &
          (year, doy, phenology, gpp, npp, C3g_no, C4g_no, &
          pool_litter, pool_som_int, pool_som_slow, &
          type_grass, gmass_leaf, gmass_root, gmass_available, &
          gmass_stock, lai_grass, lai_opt_grass)
     
END IF

!Monthly output
IF ( dom == Day_in_month(Month(doy)) ) then
     Call output_cflux &
          (year, doy, c_uptake, c_emission, pool_litter, pool_som_int, pool_som_slow, &
           mass_leaf, mass_trunk, mass_root, mass_stock, mass_available, &
           gmass_leaf, gmass_root, gmass_available, gmass_stock, type_grass)
     
     Call output_wflux &
          (year, doy, &
           hydRecord_preci, hydRecord_runoff, hydRecord_ic, hydRecord_ev, hydRecord_tr)
     
     Call output_ld_vertical &
          (year, doy, counter, tree_exist, pft, height, bole, la)
     
END IF

!Annualy output
IF (doy == 365) then
     Call output_annual &
        (year, gpp_annual, npp_annual, &
        mass_leaf, mass_trunk, mass_root, mass_stock, mass_available, &
        dbh_heartwood, dbh_sapwood, tree_exist, &
        gmass_leaf, gmass_root, gmass_stock, gmass_available, &
        type_grass, biome, &
        lai_max_wood, lai_max_grass, lai_mean_wood, lai_mean_grass, &
        pool_litter, pool_som_int, pool_som_slow, &
        la, lai_grass, canopy_cond)
    
    Call output_forest &
        (year, height, bole, dbh_heartwood, dbh_sapwood, crown_diameter, &
        bole_x, bole_y, crown_x, crown_y, pft, tree_exist)
    
    Call output_individual &
       ( year, height(1), bole(1), dbh_heartwood(1), dbh_sapwood(1), crown_diameter(1), &
       la(1), mass_leaf(1), mass_trunk(1), mass_root(1), age(1), pft(1) )
    
END IF

IF (doy == 200) then
     Call output_biomass &
        (mass_leaf, mass_trunk, mass_root, mass_stock, mass_available, tree_exist, pft, &
        gmass_leaf, gmass_root, gmass_stock, gmass_available, type_grass, C3g_no, C4g_no)
END IF


END SUBROUTINE output_point



!**************************************************************************************************
! Output individual properties
!**************************************************************************************************
SUBROUTINE output_individual &
      (year, height, bole, dbh_heartwood, dbh_sapwood, crown_diameter, &
      la, mass_leaf, mass_trunk, mass_root, age, pft)

USE data_structure
implicit none

!Define arguments

integer,intent(IN)::year           !year (1-Simulation_year)
integer,intent(IN)::age            !tree age (year)
integer,intent(IN)::height         !tree height from 1.3m above ground (STEP)
integer,intent(IN)::bole           !bole length (STEP)
integer,intent(IN)::pft            !Plant Functional Type (1-PFT_no)
real   ,intent(IN)::dbh_heartwood  !heartwood diameter at 1.3m   (m)
real   ,intent(IN)::dbh_sapwood    !sapwood diameter at 1.3m   (m)
real   ,intent(IN)::crown_diameter !crown diameter (m)
real   ,intent(IN)::la             !leaf area (m2)
real   ,intent(IN)::mass_leaf      !biomass of foliage (g)
real   ,intent(IN)::mass_trunk     !biomass of stem (g)
real   ,intent(IN)::mass_root      !biomass of root (g)

!Write title
if (year == 1) then
write ( File_no(11),* ) ' Yr age he(m) bo(m)  dia_H  dia_S  dia_C  LA    mas_F  mas_S  mas_R'
endif

!Write data
write (File_no(11),'(2i4, 2f6.1, 2f7.3, 1f7.2, 2f6.1, 3f7.1)') &
   year             ,&   !integer
   age              ,&   !integer
   height*STEP+1.3  ,&   !real
   bole*STEP+1.3    ,&   !real
   dbh_heartwood    ,&   !real
   dbh_sapwood      ,&   !real
   crown_diameter   ,&   !real
   la               ,&   !real
   mass_leaf/1000   ,&   !real
   mass_trunk/1000  ,&   !real
   mass_root/1000        !real

END SUBROUTINE output_individual



!**************************************************************************************************
! Forest status at the every 31 December
!**************************************************************************************************
SUBROUTINE output_annual &
           (year, gpp_annual, npp_annual, &
           mass_leaf, mass_trunk, mass_root, mass_stock, mass_available, &
           dbh_heartwood, dbh_sapwood, tree_exist, &
           gmass_leaf, gmass_root, gmass_stock, gmass_available, &
           type_grass, biome, &
           lai_max_wood, lai_max_grass, lai_mean_wood, lai_mean_grass, &
           pool_litter, pool_som_int, pool_som_slow, &
           la, lai_grass, canopy_cond)

USE data_structure
implicit none

!_____________ Define arguments
integer                 ,intent(IN)::year       !year

real,dimension(PFT_no)     ,intent(IN)::gpp_annual !GPP for each pft (g dm / stand / year)
real,dimension(PFT_no)     ,intent(IN)::npp_annual !NPP for each pft (g dm / stand / year)

real   ,dimension(Max_no),intent(IN)::mass_leaf      !biomass of foliage (g)
real   ,dimension(Max_no),intent(IN)::mass_trunk     !biomass of stem    (g)
real   ,dimension(Max_no),intent(IN)::mass_root      !biomass of root    (g)
real   ,dimension(Max_no),intent(IN)::mass_stock     !biomass for stock  (g)
real   ,dimension(Max_no),intent(IN)::mass_available !biomass available  (g)
real   ,dimension(Max_no),intent(IN)::dbh_heartwood  !heartwood diameter at 1.3m   (m)
real   ,dimension(Max_no),intent(IN)::dbh_sapwood    !sapwood   diameter at 1.3m   (m)
logical,dimension(Max_no),intent(IN)::tree_exist     !tree exist flag

real,dimension(2),intent(IN)::gmass_leaf      !grass foliage    biomass (g/m2)
real,dimension(2),intent(IN)::gmass_root      !grass root       biomass (g/m2)
real,dimension(2),intent(IN)::gmass_stock     !grass stock      biomass (g/m2)
real,dimension(2),intent(IN)::gmass_available !grass available  biomass (g/m2)
integer            ,intent(IN)::type_grass      !grass type priority (3:C3, 4:C4)

real,intent(IN)::lai_max_wood          !Wood  LAI @ annual maximum  (m2/m2)
real,intent(IN)::lai_max_grass         !Grass LAI @ annual maximum  (m2/m2)
real,intent(IN)::lai_mean_wood         !Wood  LAI @ annual mean     (m2/m2)
real,intent(IN)::lai_mean_grass        !Grass LAI @ annual mean     (m2/m2)

real,intent(IN)::pool_litter   !litter pool in a stand (g dm / stand)
real,intent(IN)::pool_som_int  !pool of s.o.m. with intermediated decomposition rate (g dm / stand)
real,intent(IN)::pool_som_slow !pool of s.o.m. with slow decomposition rate (g dm / stand)

real,dimension(Max_no),intent(IN)::la         !leaf area      (m2)
real,dimension(2)       ,intent(IN)::lai_grass  !c3/c4 leaf area index of grass (m2/m2)

real,intent(IN)::canopy_cond  !monthly stomatal conductance   (mol H2O m-2 s-1)

integer,intent(IN)::biome !biome type number (see subroutine biome_type for definition)

!_____________ Define local variables
real    mass_wood      !wood  biomass (g C / m2)
real    mass_grass     !grass biomass (g C / m2)
real    mass_soil      !soil  drymass (g C / m2)
real    gpp, npp       !gpp and npp (MgC/ha/year)
real    density        !tree density / above 5cm dbh (N/ha)
real    mass_mean      !mean biomass of tree / above 5cm dbh (kg)
integer i, j, no       !for temporal usage

!_____________ Prepare output variables ***
!mass wood
mass_wood = 0.0
do no=1, Max_no
if (tree_exist(no)) then
   mass_wood = mass_wood + &
   mass_leaf(no) + mass_trunk(no) + mass_root(no) + mass_stock(no) + mass_available(no)
endif
end do
mass_wood  = mass_wood * C_in_drymass / Max_loc / Max_loc /100.0

!mass grass
if (type_grass==3) then
 mass_grass = &
  0.9*(gmass_leaf(1)+gmass_root(1)+gmass_stock(1)+gmass_available(1))+ &
  0.1*(gmass_leaf(2)+gmass_root(2)+gmass_stock(2)+gmass_available(2))
else
 mass_grass = &
  0.1*(gmass_leaf(1)+gmass_root(1)+gmass_stock(1)+gmass_available(1))+ &
  0.9*(gmass_leaf(2)+gmass_root(2)+gmass_stock(2)+gmass_available(2))
endif
mass_grass = mass_grass * C_in_drymass / 100.0

!mass soil
mass_soil = pool_litter + pool_som_int + pool_som_slow
mass_soil = mass_soil * C_in_drymass / Max_loc / Max_loc / 100.0

!gpp
gpp  = sum(gpp_annual) * C_in_drymass / Max_loc / Max_loc / 100.0

!npp
npp  = sum(npp_annual) * C_in_drymass / Max_loc / Max_loc / 100.0

!density and mass_mean
j = 0
mass_mean=0.0
do no=1, Max_no
if ( tree_exist(no)                        ) then
if ( dbh_heartwood(no) + dbh_sapwood(no) > 0.05 ) then
      j = j+1
      mass_mean = mass_mean + &
      mass_leaf(no) + mass_trunk(no) + mass_root(no) + mass_stock(no) + mass_available(no)
endif
endif
end do
density = real(j) * ( (100.0/Max_loc)**2 )
mass_mean = mass_mean / real(max(1,j)) / 1000.0

!*** write title and data ***
   if (year == 1) then
      write (File_no(2), *) &
      ' wood  grass   soil   gpp   npp  LAImax_w&g  LAImen_w&g  ccon  type F_tree'
      write (File_no(2), *) &
      ' |----(MgC/ha)-----| (MgC/ha/yr) |-------(m2/m2)-------|              N/ha'
   end if
   
   write (File_no(2),'( 3(f6.1,a), 2(f5.1,a), 4(f5.1,a), 1(f6.2,a), 1(i3,a), 1(f7.1,a) )') &
   mass_wood     ,',', &
   mass_grass    ,',', &
   mass_soil     ,',', &
   gpp           ,',', &
   npp           ,',', &
   lai_max_wood  ,',', &
   lai_max_grass ,',', &
   lai_mean_wood ,',', &
   lai_mean_grass,',', &
   canopy_cond   ,',', &
   biome         ,',', &
   density              
   
END SUBROUTINE output_annual



!**************************************************************************************************
! Output forest 3D structure
!**************************************************************************************************
SUBROUTINE output_forest &
      (year, height, bole, dbh_heartwood, dbh_sapwood, crown_diameter, &
      bole_x, bole_y, crown_x, crown_y, pft, tree_exist)

USE data_structure
implicit none

!Define arguments
integer,intent(IN)::year                               !year (1-Simulation_year)
integer,dimension(Max_no),intent(IN)::height         !tree height from 1.3m above ground (STEP)
integer,dimension(Max_no),intent(IN)::bole           !bole length (STEP)
real   ,dimension(Max_no),intent(IN)::dbh_heartwood  !heartwood diameter at 1.3m   (m)
real   ,dimension(Max_no),intent(IN)::dbh_sapwood    !sapwood diameter at 1.3m   (m)
real   ,dimension(Max_no),intent(IN)::crown_diameter !crown diameter (m)
real   ,dimension(Max_no),intent(IN)::bole_x         !x_location in forest stand (m)
real   ,dimension(Max_no),intent(IN)::bole_y         !y_location in forest stand (m)
real   ,dimension(Max_no),intent(IN)::crown_x        !x_location of crown (m)
real   ,dimension(Max_no),intent(IN)::crown_y        !y_location of crown (m)
integer,dimension(Max_no),intent(IN)::pft            !Plant Functional Type (1-PFT_no)
logical,dimension(Max_no),intent(IN)::tree_exist     !tree exist or not (.TRUE. or .FALSE.)

!Define local variable
integer count, i

!Count tree number
   count = 0
   Do i=1, Max_no
      if ( tree_exist(i) ) count = count +1
   End do
   
!Write data
   write (File_no(3),'(i4,a1)') count,','
   Do i=1, Max_no
   if (tree_exist(i)) then
      write (File_no(3),'( 4(f6.1,a), 2(f7.3,a), 2(f7.4,a), i2,a )') &
      bole_x(i)                           ,',',& !1: bole location x
      bole_y(i)                           ,',',& !2: bole location y
      crown_x(i)                          ,',',& !3: crown location x
      crown_y(i)                          ,',',& !4: crown location y
      real(bole(i)) * STEP + 1.3          ,',',& !5: bole height
      real( height(i) - bole(i) ) * STEP  ,',',& !6: foliage height
      dbh_heartwood(i) + dbh_sapwood(i)   ,',',& !7: dbh
      crown_diameter(i)/2.0               ,',',& !8: crown_radius
      pft(i)                              ,','   !9: pft
      
   end if
   End do
   
   !For separate files
   !character :: file_name*32
   !write(file_name,*) year
   !file_name = trim(file_name) // '.txt'
   !open (File_no(3), file=file_name)
   !close (File_no(3))
   
END SUBROUTINE output_forest



!**************************************************************************************************
! Output air variables
!**************************************************************************************************
SUBROUTINE output_air &
      (year, doy, ap, vp, vpd, dnsa, vp_sat, slope_vps)

USE data_structure
implicit none

!Define arguments
integer,intent(IN)::year   !year (1-Simulation_year)
integer,intent(IN)::doy    !day of year (1-365)
real,intent(IN)::ap        !air pressure (hPa)
real,intent(IN)::vp        !vapour pressure (hPa)
real,intent(IN)::vpd       !vapour pressure  deficit (hPa)
real,intent(IN)::dnsa      !density of air (kg m-3)
real,intent(IN)::vp_sat    !saturation vapour pressure (hPa)
real,intent(IN)::slope_vps !slope of saturation vapour pressure related to tempertaure (hPa deg C-1)
!Write title
   if (year == 1 .and. doy==1) then
      write ( File_no(4),* ) '  Yr  doy  ap    vps      vp      vpd    dnsa  slope_vps'
   end if
   
!Write data
   write (File_no(4),'(2i4, 1f8.1, 5f8.3, f8.2)') &
      year      ,&   !integer
      doy       ,&   !integer
      
      ap        ,&   !real
      
      vp_sat    ,&   !real
      vp        ,&   !real
      vpd       ,&   !real
      
      dnsa      ,&   !real
      slope_vps      !real

END SUBROUTINE output_air



!**************************************************************************************************
! Output climate variables
!**************************************************************************************************
SUBROUTINE output_climate &
      (year, doy, tmp_air, tmp_soil, cloud, prec, humid, wind, vp, vp_sat )

USE data_structure
implicit none

!Define arguments
integer,intent(IN)::year
integer,intent(IN)::doy

real,intent(IN)::tmp_air   ! 2m air temperature (Celcius)
real,intent(IN)::tmp_soil  ! soil temperature at 10 cm  depth (Celcius)
real,intent(IN)::cloud     ! total cloudness (fraction)
real,intent(IN)::prec      ! precipitation (mm day-1)
real,intent(IN)::humid     ! specific humidity (kg kg-1)
real,intent(IN)::wind      ! wind velocity (m s-1)

real,intent(IN)::vp        !vapour pressure (hPa)
real,intent(IN)::vp_sat    !saturation vapour pressure (hPa)


!Write title
   if (year == 1 .and. doy==1) then
      write ( File_no(5),* ) '  yr  doy    air   soil  cloud    prec   humid  vp/vpsat   wind'
   end if
   
!Write data
   write (File_no(5),'(2i5, 3f7.1, f8.2, f8.3, f8.3, f7.1)') &
   year        ,& !year
   doy         ,& !day of the year
   tmp_air     ,& !2m air temperature (Celcius)
   tmp_soil    ,& !soil temperature at 10 cm  depth (Celcius)
   cloud       ,& !total cloudness (fraction)
   prec        ,& !precipitation (mm day-1)
   humid       ,& !specific humidity (kg kg-1)
   vp/vp_sat   ,& !relative humidity (proportion)
   wind           !wind  velocity (m s-1)
   
END SUBROUTINE output_climate



!**************************************************************************************************
! Output air variables
!**************************************************************************************************
SUBROUTINE output_radiation &
      (year, doy, sl_hgt, dlen, rad, par, par_grass)

USE data_structure
implicit none

!Define arguments
integer,intent(IN)::year   !year (1-Simulation_year)
integer,intent(IN)::doy    !day of year (1-365)
real,intent(IN)::sl_hgt    !solar hight at midday (degree)
real,intent(IN)::dlen      !day length (hour)
real,intent(IN)::rad       !global radiation at the ground surface (W/m2)
real,intent(IN)::par       !canopy top PAR of mid-day (micro mol photon m-2 s-1)
real,intent(IN)::par_grass !under canopy PAR of mid-day (micro mol photon m-2 s-1)

!Write title
   if (year == 1 .and. doy==1) then
      write ( File_no(6),* ) ' Yr doy  s_hgt dlen  rad    par     par_grass'
   end if
   
!Write data
   write (File_no(6),'(2i4, 2f6.1, 1f7.1, 2f8.1)') &
      year      ,& !integer
      doy       ,& !integer
      sl_hgt    ,& !real (degree)
      dlen      ,& !real (hour)
      rad       ,& !real (W/m2)
      par       ,& !real (micro mol photon m-2 s-1)
      par_grass    !real (micro mol photon m-2 s-1)

END SUBROUTINE output_radiation



!**************************************************************************************************
! Output net radiation
!**************************************************************************************************
SUBROUTINE output_netradiation &
      (year, doy, albedo_soil, albedo_veg, &
      radnet_soil, radnet_veg)

USE data_structure
implicit none

!Define arguments
integer,intent(IN)::year     !year (1-Simulation_year)
integer,intent(IN)::doy      !day of year (1-365)
real,intent(IN)::albedo_soil !soil surface albedo
real,intent(IN)::albedo_veg  !vegetation area albedo
real,intent(IN)::radnet_soil !soil surface net radiation, short wave (W m-2)
real,intent(IN)::radnet_veg  !canopy net radiation, short wave (W m-2)

!Write title
   if (year == 1 .and. doy==1) then
      write ( File_no(7),* ) ' Yr doy  s_alb  v_alb  radnet_soil radnet_veg'
   end if
   
!Write data
   write (File_no(7),'(2i4, 2f7.2, 2f10.1)') &
      year        ,&   !integer
      doy         ,&   !integer
      
      albedo_soil ,&   !real
      albedo_veg  ,&   !real
      radnet_soil ,&   !real
      radnet_veg       !real

END SUBROUTINE output_netradiation



!**************************************************************************************************
! Output water status
!**************************************************************************************************
SUBROUTINE output_water &
      (year, doy, pool_snow, pool_w1, pool_w2, W_fi, W_wilt, W_sat, W_mat)

USE data_structure
implicit none

!Define arguments
integer,intent(IN)::year      !year (1-Simulation_year)
integer,intent(IN)::doy       !day of year (1-365)
real   ,intent(IN)::pool_w1   !soil water content of soil layer 1 (mm)
real   ,intent(IN)::pool_w2   !soil water content of soil layer 2 (mm)
real   ,intent(IN)::pool_snow !water equivalent snow depth (mm)

real,intent(IN)::W_fi         !filed capacity   (m3/m3, 0.0 -> 1.0)
real,intent(IN)::W_wilt       !wilting point    (m3/m3, 0.0 -> 1.0)
real,intent(IN)::W_sat        !saturate point   (m3/m3, 0.0 -> 1.0)
real,intent(IN)::W_mat        !matrix potential (m, -0.0001 -> -3.0)

!Define local valuable
real a1, a2, x

!Prepare values
!a1 = pool_w1 / (W_sat * Depth1)
!a2 = pool_w2 / (W_sat * Depth2)

a1 = (pool_w1/Depth1-W_wilt) / (W_fi-W_wilt)
a1 = max(0.0, min(1.0, a1))

a2 = (pool_w2/Depth2-W_wilt) / (W_fi-W_wilt)
a2 = max(0.0, min(1.0, a2))

!Write title
   if (year == 1 .and. doy==1) then
      write ( File_no(8),* ) 'Yr doy   snow   pool_w1   pool_w2  stat_w1   stat_w2'
   end if

!Write data
   write (File_no(8),'(2i4, 5f10.2, 2f10.3)') &
      year      ,&   !integer
      doy       ,&   !integer
      pool_snow ,&
      pool_w1   ,&
      pool_w2   ,&
      a1        ,&
      a2          

END SUBROUTINE output_water



!**************************************************************************************************
! Output carbon flux status
!**************************************************************************************************
SUBROUTINE output_cflux &
          (year, doy, c_uptake, c_emission, pool_litter, pool_som_int, pool_som_slow, &
           mass_leaf, mass_trunk, mass_root, mass_stock, mass_available, &
           gmass_leaf, gmass_root, gmass_available, gmass_stock, type_grass)

USE data_structure
implicit none

!Define arguments
integer,intent(IN)::year       !year  (1-Simulation_year)
integer,intent(IN)::doy        !day of year (1-365)
real   ,intent(IN)::c_emission !monthly carbon emission (C g /month/stand)
real   ,intent(IN)::c_uptake   !monthly carbon uptake   (C g /month/stand)

real   ,intent(IN)::pool_litter   !Litter in a stand (g drymatter / stand)
real   ,intent(IN)::pool_som_int  !SOM with intermediated decomposition rate (g dm/stand)
real   ,intent(IN)::pool_som_slow !SOM with slow decomposition rate (g dm/stand)

real,dimension(Max_no),intent(IN)::mass_leaf       !biomass of foliage (g DM)
real,dimension(Max_no),intent(IN)::mass_trunk      !biomass of stem    (g DM)
real,dimension(Max_no),intent(IN)::mass_root       !biomass of root    (g DM)
real,dimension(Max_no),intent(IN)::mass_stock      !biomass for stock  (g DM)
real,dimension(Max_no),intent(IN)::mass_available  !biomass available  (g DM)

real,dimension(2)     ,intent(IN)::gmass_leaf      !biomass of grass foliage   (g DM/m2)
real,dimension(2)     ,intent(IN)::gmass_root      !biomass of grass root      (g DM/m2)
real,dimension(2)     ,intent(IN)::gmass_available !biomass of grass available (g DM/m2)
real,dimension(2)     ,intent(IN)::gmass_stock     !biomass of grass stock     (g DM/m2)
integer                 ,intent(IN)::type_grass      !grass type priority (3:C3, 4:C4)

!Define local valiables
real a1, a2, a3
real b1, b2, b3
real c1, c2

!Prepare output values
   a1 = c_uptake   /Max_loc/Max_loc / 100.0 !carbon uptake (Mg C/month/ha)
   a2 = c_emission /Max_loc/Max_loc / 100.0 !carbon uptake (Mg C/month/ha)
   a3 = a1 - a2                             !NEP           (Mg C/month/ha)
   
   b1 = pool_litter   * C_in_drymass / Max_loc / Max_loc / 100.0
   b2 = pool_som_int  * C_in_drymass / Max_loc / Max_loc / 100.0
   b3 = pool_som_slow * C_in_drymass / Max_loc / Max_loc / 100.0
   
   c1 = sum(mass_leaf) + sum(mass_trunk) + sum(mass_root) + sum(mass_stock) + sum(mass_available)
   c1 = c1 * C_in_drymass / Max_loc / Max_loc / 100.0
   
   if (type_grass==3) then
   c2 = 0.9 * (gmass_leaf(1)+gmass_root(1)+gmass_available(1)+gmass_stock(1)) + &
        0.1 * (gmass_leaf(2)+gmass_root(2)+gmass_available(2)+gmass_stock(2))    
   else
   c2 = 0.1 * (gmass_leaf(1)+gmass_root(1)+gmass_available(1)+gmass_stock(1)) + &
        0.9 * (gmass_leaf(2)+gmass_root(2)+gmass_available(2)+gmass_stock(2))    
   end if
   c2 = c2 * C_in_drymass / 100.0

!Write title
if (year == 1 .and. Month(doy)==1) then
   write (File_no(9),*) &
         '       |----- Unit = Mg C ha-1 -------------------------------------->|'
   write (File_no(9),*) &
         'Yr  mon uptake  emiss    NEP   litter  SOM_int SOM_slow  mass_w mass_g'
end if

!Write data
write (File_no(9),'( 1(i4,a), 1(i2,a), 3(f6.1,a), 4(f8.1,a), 1(f6.1) )') &
      year       ,',', &
      Month(doy) ,',', &
      a1         ,',', &
      a2         ,',', &
      a3         ,',', &
      b1         ,',', &
      b2         ,',', &
      b3         ,',', &
      c1         ,',', &
      c2                

END SUBROUTINE output_cflux



!**************************************************************************************************
! Output water flux status
!**************************************************************************************************
SUBROUTINE output_wflux &
          (year, doy, hydRecord_preci, hydRecord_runoff, hydRecord_ic, hydRecord_ev, hydRecord_tr)

USE data_structure
implicit none

!Define arguments
integer,intent(IN)::year       !year  (1-Simulation_year)
integer,intent(IN)::doy        !day of year (1-365)

!Hydrogy record
real,intent(IN):: hydRecord_preci  !sum of precipitation      (mm/month)
real,intent(IN):: hydRecord_runoff !sum of runoff water       (mm/month)
real,intent(IN):: hydRecord_ic     !sum of interception water (mm/month)
real,intent(IN):: hydRecord_ev     !sum of evaporated water   (mm/month)
real,intent(IN):: hydRecord_tr     !sum of tranpirated water  (mm/month)

!Write title
if (year == 1 .and. Month(doy)==1) then
   write (File_no(14),*) &
         '         |----- Unit = mm month-1 ----------------->|'
   write (File_no(14),*) &
         ' Yr  mon     prec       RO       IC       EV       TR'
end if

!Write data
write (File_no(14),'( 1(i5,a), 1(i3,a), 5(f8.2,a) )') &
      year                ,',', &
      Month(doy)          ,',', &
      hydRecord_preci     ,',', &
      hydRecord_runoff    ,',', &
      hydRecord_ic        ,',', &
      hydRecord_ev        ,',', &
      hydRecord_tr               

END SUBROUTINE output_wflux


!**************************************************************************************************
! Output water flux status
!o͂PFTɉāAR[hύXKv܂
!**************************************************************************************************
SUBROUTINE output_ld_vertical &
           (year, doy, counter, tree_exist, pft, height, bole, la)

USE data_structure
implicit none

!_________________ Define local parameters
integer,parameter::Fn                = 17 !file number to write result
integer,parameter::Vertical_Interval = 10 !number of STEP for each vertical layer for output

!_________________ Define augments
integer                    ,intent(IN)::year       !loop counter for year (1-Simulation_year)
integer                    ,intent(IN)::doy        !day of the year (1-365)
integer                    ,intent(IN)::counter    !simulation day (1-Simulation_year*365)
logical,dimension(Max_no),intent(IN)::tree_exist !tree exist flag
integer,dimension(Max_no),intent(IN)::pft        !Plant Functional Type
integer,dimension(Max_no),intent(IN)::height     !tree height from 1.3m above ground (STEP)
integer,dimension(Max_no),intent(IN)::bole       !bole length from 1.3m above ground (STEP)
real   ,dimension(Max_no),intent(IN)::la         !leaf area           (m2)

!_________________ Define local variables
integer        no, p, l1, l2
real           x 
real,dimension(PFT_no, Max_hgt )                       ::ld1 !sum of leaf area for each step
real,dimension(PFT_no, int(Max_hgt/Vertical_Interval) )::ld2 !sum of leaf area for each interval

!_________________ Sumup leaf area for each tree
ld1(:,:) = 0.0
Do no=1, Max_no
if ( .not. tree_exist(no) ) cycle
   p = pft(no)
   x = la(no) / real(height(no) - bole(no)) !leaf area for each crown disk
   do l1 = bole(no)+1, height(no)
      ld1(p,l1) = ld1(p,l1) + x
   end do
End do
ld1(:,:) = ld1(:,:) / Max_loc / Max_loc !adjust unit: (m2 step-1 stand-1) --> (m2 step-1 m-2)

!_________________ Convert vertical resolution
ld2(:,:) = 0.0
Do p=1, PFT_no
Do l1=1, Max_hgt
   l2 = int(l1/Vertical_Interval) + 1
   ld2(p,l2) = ld2(p,l2) + ld1(p,l1)
End do
End do

!_________________ Write Time stamp
!write (File_no(Fn), '( 1(i5,a),1(i5,a),1(i7,a) )') year,',',doy,',',counter

!_________________ Write distribution
write (File_no(Fn), *)
Do l2=1, int(Max_hgt/Vertical_Interval) + 1
!   write (File_no(Fn),'( f8.5,a,f8.5,a,f8.5,a,f8.5 )') &
!         ( ld2(1,l2),',',ld2(2,l2),',',ld2(3,l2),',',ld2(4,l2) )
End do

END SUBROUTINE output_ld_vertical



!**************************************************************************************************
! Output grass
!**************************************************************************************************
SUBROUTINE output_grass &
      (year, doy, phenology, gpp, npp, C3g_no, C4g_no, &
      pool_litter, pool_som_int, pool_som_slow, &
      type_grass, gmass_leaf, gmass_root, gmass_available, &
      gmass_stock, lai_grass, lai_opt_grass)

USE data_structure
implicit none

!_________________ Define arguments
integer,intent(IN)::year !year (1-Simulation_year)
integer,intent(IN)::doy  !day of year (1-365)

real,intent(IN)::pool_litter   !litter pool in a stand (g drymatter / stand)
real,intent(IN)::pool_som_int  !pool of s.o.m. with intermediated decomposition rate (g dm/stand)
real,intent(IN)::pool_som_slow !pool of s.o.m. with slow decomposition rate (g dm / stand)

logical,dimension(PFT_no),intent(IN)::phenology !Phenological status (T:foliation, F:dormance)
real   ,dimension(PFT_no),intent(IN)::gpp       !daily GPP for each pft (g drymatter m-2 day-1)
real   ,dimension(PFT_no),intent(IN)::npp       !daily NPP for each pft (g drymatter m-2 day-1)

integer,intent(IN)::C3g_no !PFT number of C3 grass
integer,intent(IN)::C4g_no !PFT number of C4 grass

integer            ,intent(IN)::type_grass      !grass type priority (3:C3 type, 4:C4 type)
real,dimension(2),intent(IN)::gmass_leaf      !grass foliage    biomass (g/m2)
real,dimension(2),intent(IN)::gmass_root      !grass root       biomass (g/m2)
real,dimension(2),intent(IN)::gmass_available !grass available  biomass (g/m2)
real,dimension(2),intent(IN)::gmass_stock     !grass stock      biomass (g/m2)
real,dimension(2),intent(IN)::lai_grass       !leaf area index of grass   (m2/m2)
real,dimension(2),intent(IN)::lai_opt_grass   !optimum LAI of grass layer (m2/m2)

!_________________ Define local variable
real      a1, a2
character phenology_symbol_c3, phenology_symbol_c4
real      mass_f_ave, mass_r_ave, mass_stock_ave
real      mass_total_ave, mass_soil_ave, lai_ave, lai_opt_ave

!_________________ set phenology_symbol
phenology_symbol_c3='-' ; if (phenology(C3g_no)) phenology_symbol_c3='*'
phenology_symbol_c4='-' ; if (phenology(C4g_no)) phenology_symbol_c4='*'

!_________________ prepare output variables
  a1 = gmass_leaf(1) + gmass_root(1) + gmass_available(1) + gmass_stock(1)
  a2 = gmass_leaf(2) + gmass_root(2) + gmass_available(2) + gmass_stock(2)
  
if (type_grass==3) then
  mass_total_ave = 0.9*a1              + 0.1*a2
  mass_f_ave     = 0.9*gmass_leaf(1)   + 0.1*gmass_leaf(2)
  mass_r_ave     = 0.9*gmass_root(1)   + 0.1*gmass_root(2)
  mass_stock_ave = 0.9*gmass_stock(1)  + 0.1*gmass_stock(2)
  lai_ave        = 0.9*lai_grass(1)    + 0.1*lai_grass(2)
  lai_opt_ave    = 0.9*lai_opt_grass(1)+ 0.1*lai_opt_grass(2)
else
  mass_total_ave = 0.1*a1              + 0.9*a2
  mass_f_ave     = 0.1*gmass_leaf(1)   + 0.9*gmass_leaf(2)
  mass_r_ave     = 0.1*gmass_root(1)   + 0.9*gmass_root(2)
  mass_stock_ave = 0.1*gmass_stock(1)  + 0.9*gmass_stock(2)
  lai_ave        = 0.1*lai_grass(1)    + 0.9*lai_grass(2)
  lai_opt_ave    = 0.1*lai_opt_grass(1)+ 0.9*lai_opt_grass(2)
end if
  
  mass_total_ave = C_in_drymass * mass_total_ave
  mass_soil_ave  = C_in_drymass * (pool_litter+pool_som_int+pool_som_slow) / Max_loc / Max_loc

!_________________ Write title
if (year==1 .and. doy==1) then
   write ( File_no(12),* ) 'Yr doy pt C3   F    R    S  lai lai_o  C4   F    R    S  lai lai_o'
end if

!_________________ Write data
   write (File_no(12),'( i3 , i4 , i3 , 2(a3, 3f5.0, 2f5.1) )') &
    year, doy, type_grass,  &
    
    phenology_symbol_c3, gmass_leaf(1), gmass_root(1), gmass_stock(1), &
    lai_grass(1), lai_opt_grass(1), &
    
    phenology_symbol_c4, gmass_leaf(2), gmass_root(2), gmass_stock(2), &
    lai_grass(2), lai_opt_grass(2)

END SUBROUTINE output_grass



!**************************************************************************************************
! Biomass composition
!**************************************************************************************************
SUBROUTINE output_biomass &
        (mass_leaf, mass_trunk, mass_root, mass_stock, mass_available, tree_exist, pft, &
        gmass_leaf, gmass_root, gmass_stock, gmass_available, type_grass, C3g_no, C4g_no )

USE data_structure
implicit none

!*** Define arguments ***
real   ,dimension(Max_no),intent(IN)::mass_leaf      !biomass of foliage (g DM)
real   ,dimension(Max_no),intent(IN)::mass_trunk     !biomass of stem    (g DM)
real   ,dimension(Max_no),intent(IN)::mass_root      !biomass of root    (g DM)
real   ,dimension(Max_no),intent(IN)::mass_stock     !biomass for stock  (g DM)
real   ,dimension(Max_no),intent(IN)::mass_available !biomass available  (g DM)
integer,dimension(Max_no),intent(IN)::pft            !Plant Functional Type (1-PFT_no)
logical,dimension(Max_no),intent(IN)::tree_exist     !tree exist flag

real,dimension(2),intent(IN)::gmass_leaf      !grass foliage   biomass (g DM/m2)
real,dimension(2),intent(IN)::gmass_root      !grass root      biomass (g DM/m2)
real,dimension(2),intent(IN)::gmass_stock     !grass stock     biomass (g DM/m2)
real,dimension(2),intent(IN)::gmass_available !grass available biomass (g DM/m2)
integer            ,intent(IN)::type_grass      !grass type priority (3:C3, 4:C4)
integer            ,intent(IN)::C3g_no          !PFT number of C3 grass
integer            ,intent(IN)::C4g_no          !PFT number of C4 grass

!*** Define local variables ***
real,dimension(PFT_no)::biomass !biomass for each PFT (g C / m2)
integer                 no      !for loop counter
real                    x, y    !for gengeral usage

!*** reset output variables ***
   biomass(:)=0.0
   
!*** woody biomass (g dm / stand) ***
   Do no=1, Max_no
   If (tree_exist(no)) then
      biomass(pft(no)) = biomass(pft(no))+&
      mass_leaf(no)+mass_trunk(no)+mass_root(no)+mass_stock(no)+mass_available(no)
   End if
   End do
   
!*** grass biomass (g dm / stand) ***
   x = gmass_leaf(1) + gmass_root(1) + gmass_stock(1) + gmass_available(1)
   y = gmass_leaf(2) + gmass_root(2) + gmass_stock(2) + gmass_available(2)
   
   if (type_grass==3) then
      biomass(C3g_no) = 0.9 * Max_loc * Max_loc * x
      biomass(C4g_no) = 0.1 * Max_loc * Max_loc * y
   else
      biomass(C3g_no) = 0.1 * Max_loc * Max_loc * x
      biomass(C4g_no) = 0.9 * Max_loc * Max_loc * y
   endif
   
!*** adjust unit ( g dm / stand ---> Mg C / ha ) ***
   biomass(:) = biomass(:) * C_in_drymass / Max_loc / Max_loc / 100.0
   
!*** write title and data ***
   write (File_no(13),'( 13(f6.1,1x) )') biomass(:)
   
END SUBROUTINE output_biomass



!*************************************************************************************************
! Output for Viewer
!*************************************************************************************************
SUBROUTINE output_for_viewer &
   (LAT, LON, ALT, Albedo_soil0, W_fi, W_wilt, W_sat, W_mat, &
   biome, c_emission, c_uptake, pool_litter, pool_som_int, pool_som_slow, &
   counter, year, doy, &
   tmp_air, tmp_soil, cloud, prec, humid, wind, &
   co2atm, ap, vp, vp_sat, dnsa, slope_vps, &
   sl_hgt, dlen, par, par_direct, par_diffuse, par_grass, &
   rad, radnet_soil, radnet_veg, albedo_soil, albedo_veg, ir_tree, ir_grass, &
   canopy_cond, pool_w1, pool_w2, pool_w3, pool_snow, &
   phenology, lue, co2cmp, psat, lue_grass, co2cmp_grass, psat_grass, gpp_month, npp_month, &
   tree_exist, pft, la, mass_leaf, mass_trunk, mass_root, mass_stock, mass_available, &
   dbh_heartwood, dbh_sapwood, &
   type_grass, C3g_no, C4g_no, &
   lai_grass, gmass_leaf, gmass_root, gmass_available, gmass_stock, lai_opt_grass, &
   hydRecord_preci, hydRecord_runoff, hydRecord_ic, hydRecord_ev, hydRecord_tr )

USE data_structure
implicit none
common PN_s, PN_r

!_____________ Define arguments
!PFT specific constants
real,dimension(PFT_no)::PN_s !proportion of nitrogen:stem (gN / gBiomass)
real,dimension(PFT_no)::PN_r !proportion of nitrogen:root (gN / gBiomass)

!Location data (single precision)
real,intent(IN)::LAT          !latitude  (degree)
real,intent(IN)::LON          !longitude  (degree)
real,intent(IN)::ALT          !altitude  (m above MSL)
real,intent(IN)::Albedo_soil0 !soil albedo
real,intent(IN)::W_fi         !filed capacity  (m3/m3, 0.0 -> 1.0)
real,intent(IN)::W_wilt       !wilting point   (m3/m3, 0.0 -> 1.0)
real,intent(IN)::W_sat        !saturate point  (m3/m3, 0.0 -> 1.0)
real,intent(IN)::W_mat        !matrix potential

integer,intent(IN)::biome !biome type number (see subroutine biome_type for definition)

real   ,intent(IN)::c_emission !monthly carbon emission (C g /month/stand)
real   ,intent(IN)::c_uptake   !monthly carbon uptake   (C g /month/stand)

real   ,intent(IN)::pool_litter   !Litter in a stand (g drymatter / stand)
real   ,intent(IN)::pool_som_int  !SOM with intermediated decomposition rate (g dm/stand)
real   ,intent(IN)::pool_som_slow !SOM with slow decomposition rate (g dm/stand)

!Simulation counter
integer,intent(IN)::counter !simulation day (1-Simulation_year*365)
integer,intent(IN)::year  !year  (1-Simulation_year)
integer,intent(IN)::doy   !day of year (1-365)

!Climatic data (single precision)
real,intent(IN)::tmp_air  !2m air temperature              (Celcius) 
real,intent(IN)::tmp_soil !soil temperature at 10 cm depth (Celcius) 
real,intent(IN)::cloud    !total cloudiness                (fraction)
real,intent(IN)::prec     !precipitation                   (mm day-1)
real,intent(IN)::humid    !specific humidity               (kg kg-1) 
real,intent(IN)::wind     !wind velocity                   (m s-1)   

!Physical status of air
real,intent(IN)::co2atm    !ambient (canopy) CO2 concentration (ppmv)
real,intent(IN)::ap        !air pressure               (hPa)
real,intent(IN)::vp        !vapour pressure            (hPa)
real,intent(IN)::vp_sat    !saturation vapour pressure (hPa)
real,intent(IN)::dnsa      !density of air (kg m-3)
real,intent(IN)::slope_vps !slope of saturation vapour pressure related to air temp(hPa deg C-1)

!Physical status of radiation
! (U1)...micro mol photon m-2 s-1
real,intent(IN)::sl_hgt      !solar hight at midday (degree)
real,intent(IN)::dlen        !day length (hour)
real,intent(IN)::par         !photosynthetically active radiation of mid-day (U1)
real,intent(IN)::par_direct  !direct PAR on canopy top of mid-day            (U1)
real,intent(IN)::par_diffuse !difussed PAR on canopy top of mid-day          (U1)
real,intent(IN)::par_grass   !total PAR on top of the grass layer            (U1)
real,intent(IN)::rad         !global radiation over canopy           (W m-2)
real,intent(IN)::radnet_soil !soil surface net radiation, short wave (W m-2)
real,intent(IN)::radnet_veg  !canopy net radiation, short wave       (W m-2)
real,intent(IN)::albedo_soil !soil surface albedo    (0.0-1.0)
real,intent(IN)::albedo_veg  !vegetation area albedo (0.0-1.0)
real,intent(IN)::ir_tree     !radiation-interruption-coefficient by tree corwn (0.0-1.0)
real,intent(IN)::ir_grass    !radiation-interruption-coefficient by grass leaf (0.0-1.0)

!Hydrogical varibales
real,intent(IN)::canopy_cond !canopy conductance (mol H2O m-2 s-1)
real,intent(IN)::pool_w1     !soil water content of soil layer 1 (mm)
real,intent(IN)::pool_w2     !soil water content of soil layer 2 (mm)
real,intent(IN)::pool_w3     !soil water content of soil layer 3 (mm)
real,intent(IN)::pool_snow   !water equivalent snow depth        (mm)

!PFT specific photosynthesis variables
logical,dimension(PFT_no),intent(IN)::phenology !Phenological status
real,dimension(PFT_no),intent(IN)::gpp_month    !GPP for each pft (g dm / stand / month)
real,dimension(PFT_no),intent(IN)::npp_month    !NPP for each pft (g dm / stand / month)
real,dimension(Max_no),intent(IN)::lue          !light use efficiency (mol CO2 mol photon-1)
real,dimension(Max_no),intent(IN)::co2cmp       !CO2 compensation point (ppmv)
real,dimension(Max_no),intent(IN)::psat         !light-saturated photosynthesis rate
real,dimension(2)     ,intent(IN)::lue_grass    !light use efficiency
real,dimension(2)     ,intent(IN)::co2cmp_grass !CO2 compensation point (ppmv)
real,dimension(2)     ,intent(IN)::psat_grass   !light-saturated photosynthesis rate

!Properties of woody PFTs
logical,dimension(Max_no),intent(IN)::tree_exist     !tree exist flag
integer,dimension(Max_no),intent(IN)::pft            !Plant Functional Type
real   ,dimension(Max_no),intent(IN)::la             !leaf area           (m2)
real   ,dimension(Max_no),intent(IN)::mass_leaf      !biomass of foliage (g DM)
real   ,dimension(Max_no),intent(IN)::mass_trunk     !biomass of stem    (g DM)
real   ,dimension(Max_no),intent(IN)::mass_root      !biomass of root    (g DM)
real   ,dimension(Max_no),intent(IN)::mass_stock     !biomass for stock  (g DM)
real   ,dimension(Max_no),intent(IN)::mass_available !biomass available  (g DM)
real   ,dimension(Max_no),intent(IN)::dbh_heartwood  !heartwood diameter at 1.3m (m)
real   ,dimension(Max_no),intent(IN)::dbh_sapwood    !sapwood   diameter at 1.3m (m)

!Properties of grass PFTs
integer          ,intent(IN)::type_grass      !grass type priority in this girid (3:C3, 4:C4)
integer          ,intent(IN)::C3g_no          !PFT number of C3 grass
integer          ,intent(IN)::C4g_no          !PFT number of C4 grass

real,dimension(2),intent(IN)::lai_grass       !c3/c4 leaf area index of grass (m2/m2)
real,dimension(2),intent(IN)::gmass_leaf      !c3/c4 grass foliage   biomass (g/m2)
real,dimension(2),intent(IN)::gmass_root      !c3/c4 grass root      biomass (g/m2)
real,dimension(2),intent(IN)::gmass_available !c3/c4 grass available biomass (g/m2)
real,dimension(2),intent(IN)::gmass_stock     !c3/c4 grass stock     biomass (g/m2)
real,dimension(2),intent(IN)::lai_opt_grass   !c3/c4 optimum LAI for grass  (m2/m2)

!Hydrogy record
real,intent(IN)::hydRecord_preci  !sum of precipitation      (mm/month)
real,intent(IN)::hydRecord_runoff !sum of runoff water       (mm/month)
real,intent(IN)::hydRecord_ic     !sum of interception water (mm/month)
real,intent(IN)::hydRecord_ev     !sum of evaporated water   (mm/month)
real,intent(IN)::hydRecord_tr     !sum of tranpirated water  (mm/month)

!_____________ Define local variables
character(len=3)                  PFT_no_string
real   ,dimension(PFT_no)       ::lai, tmp0, tmp1, tmp2, tmp3
real   ,dimension(PFT_no)       ::c_leaf, c_trunk, c_root, c_stock, c_available
integer,dimension(PFT_no,0:41)::size_dist

integer i, j, k, p, no, Fn
real    a1, a2, a3, a4, b1, b2, b3, b4, c1, c2, x
real    tree_density, Dist_step

!_____________ Define local parameter
Dist_step = 2.5  !D.B.H. size step for size class output
Fn = File_no(16) !IO number

!_____________ Write File header
If ( counter==1 .or.  (Flag_spinup_read .and. counter==1+Spinup_year*365) ) then
   
   !prepare strings for maximum PFT number
   i = int(  PFT_no / 100      ) ; PFT_no_string(1:1)=char(i+48)
   j = int( (PFT_no-i*100)/10  ) ; PFT_no_string(2:2)=char(j+48)
   k = int(  PFT_no-i*100-j*10 ) ; PFT_no_string(3:3)=char(k+48)
   
   !write out setting
   write (Fn, '( 3(f7.2,a), i5, a, i5, 3(a,f8.1), a,i4 )' )  &
   LAT,',',LON,',',ALT,',',Simulation_year,',',PFT_no,',',Depth1,',',Depth2,',',Depth3,',',Max_loc
   
   write (Fn, '( f7.5,a,f7.5,a,f7.5,a,f7.5,a,f7.5 )' ) &
   Albedo_soil0,',',W_fi,',',W_wilt,',',W_sat,',',W_mat
   
   do p=1, PFT_no; write (Fn,'(i1,a$)'  ) Life_type(p),     ','; end do
   write (Fn,*)
   
   do p=1, PFT_no; write (Fn,'(i1,a$)'  ) Phenology_type(p),','; end do
   write (Fn,*)
   
   do p=1, PFT_no; write (Fn,'(f6.4,a$)') PN_s(p),          ','; end do
   write (Fn,*)
   
   do p=1, PFT_no; write (Fn,'(f6.4,a$)') PN_r(p),          ','; end do
   write (Fn,*)
   
   write (Fn,*)
   
Endif

!_____________ Write Daily output
!*** 1st line (Simulation day counter)
   write (Fn,'(i7)') &
   counter

!*** 2nd line (Climatic data)
   write (Fn,'( 2(f5.1,a), f5.2,a, f6.2,a, f6.3,a, f5.2 )') &
   tmp_air,',',tmp_soil,',',cloud,',',prec,',',humid,',',wind

!*** 3rd line (Physical status of air)
   write (Fn,'( f6.1,a, 2(f8.3,a), f6.3,a, f6.3 )') &
   ap,',',vp_sat,',',vp,',',dnsa,',',slope_vps

!*** 4th (Radiation properties on vegetation surface)
   write (Fn,'( 4(f4.2,a) )') &
   albedo_soil,',',albedo_veg,',',ir_tree,',',ir_grass

!*** 5th line (Physical status of radiation)
   write (Fn,'( f6.1,a, f6.1,a, f6.1,a, f6.1,a, f6.1,a, f6.1 )') &
   par_direct,',',par_diffuse,',',par_grass,',',rad,',',radnet_soil,',',radnet_veg

!*** 6th line (Hydrogical varibales)
   write (Fn,'( f7.2, 3(a,f6.1), a,f8.1 )') &
   canopy_cond,',',pool_w1,',',pool_w2,',',pool_w3,',',pool_snow
   
!*** 7th~ lines (PFT specific variables)
   !LAI of Woody PFTs
   lai(:) = 0.0
   Do no=1, Max_no
      if ( .not. tree_exist(no) ) cycle
      p = pft(no)
      lai(p) = lai(p) + la(no)
   End do
   lai(:) = lai(:) / Max_loc / Max_loc
   
   !LAI of Grass PFTs
   if (type_grass ==3) then
      lai(C3g_no) = lai_grass(1) * 0.9
      lai(C4g_no) = lai_grass(2) * 0.1
   else
      lai(C3g_no) = lai_grass(1) * 0.1
      lai(C4g_no) = lai_grass(2) * 0.9
   endif
   
   !Write each PFT data
   tmp0(:)=0.0 ; tmp1(:)=0.0 ; tmp2(:)=0.0 ; tmp3(:)=0.0
   Do no=1, Max_no
      if ( .not. tree_exist(no)     ) cycle
      if ( .not. phenology(pft(no)) ) cycle
      if ( la(no) <=0.0             ) cycle
      
      p = pft(no)
      tmp0(p) = tmp0(p) + 1
      tmp1(p) = tmp1(p) + lue   (no)
      tmp2(p) = tmp2(p) + co2cmp(no)
      tmp3(p) = tmp3(p) + psat  (no)
   End do
   
   Do p=1, PFT_no
      tmp1(p) = tmp1(p) / Max(1, tmp0(p))
      tmp2(p) = tmp2(p) / Max(1, tmp0(p))
      tmp3(p) = tmp3(p) / Max(1, tmp0(p))
   End do
   
   tmp1(C3g_no) = lue_grass   (1) ; tmp1(C4g_no) = lue_grass   (2)
   tmp2(C3g_no) = co2cmp_grass(1) ; tmp2(C4g_no) = co2cmp_grass(2)
   tmp3(C3g_no) = psat_grass  (1) ; tmp3(C4g_no) = psat_grass  (2)
   
   Do p=1, PFT_no
      write (Fn,'( f5.2, a, f6.3, a, f7.3, a, f4.1 )') &
      lai(p),',',tmp1(p),',',tmp2(p),',',tmp3(p)
   End do
   
!_____________ Write Monthly output
IF ( Day_of_Month(doy) == Day_in_month(Month(doy)) ) then
   
!*** 1st line
   !prepare tree density (N/ha)
   i = 0
   do no=1, Max_no
      if ( tree_exist(no) ) i = i + 1
   end do
   tree_density = real(i) * ( (100.0/Max_loc)**2 )
   
   write (Fn, '(i5, a, i2, a, i2, a, f8.1, a, i2)')  &
         year,',',Month(doy),',',biome,',',tree_density,',',type_grass
   
!*** 2nd line (Carbon strage & flux)
   !Sumup carbon properties
   a1 = c_uptake   /Max_loc/Max_loc / 100.0 !carbon uptake   (Mg C/month/ha)
   a2 = c_emission /Max_loc/Max_loc / 100.0 !carbon emission (Mg C/month/ha)
   
   b1 = pool_litter   * C_in_drymass / Max_loc / Max_loc / 100.0 !Litter  (Mg C/ha)
   b2 = pool_som_int  * C_in_drymass / Max_loc / Max_loc / 100.0 !SOM int (Mg C/ha)
   b3 = pool_som_slow * C_in_drymass / Max_loc / Max_loc / 100.0 !SOM slow(Mg C/ha)
   
   !Woody carbon (Mg C/ha)
   c1 = 0.0
   Do no=1, Max_no
   If (tree_exist(no)) then
      c1 = c1 + mass_leaf(no)+mass_trunk(no)+mass_root(no)+mass_stock(no)+mass_available(no)
   End if
   End do
   c1 = c1 * C_in_drymass / Max_loc / Max_loc / 100.0
   
   !Grass carbon (Mg C/ha)
   if (type_grass==3) then
   c2 = 0.9 * (gmass_leaf(1)+gmass_root(1)+gmass_available(1)+gmass_stock(1)) + &
        0.1 * (gmass_leaf(2)+gmass_root(2)+gmass_available(2)+gmass_stock(2))    
   else
   c2 = 0.1 * (gmass_leaf(1)+gmass_root(1)+gmass_available(1)+gmass_stock(1)) + &
        0.9 * (gmass_leaf(2)+gmass_root(2)+gmass_available(2)+gmass_stock(2))    
   end if
   c2 = c2 * C_in_drymass / 100.0
   
   write (Fn, '( 2(f6.1,a), 4(f8.1,a), f6.1 )') &
         a1,',',a2,',',b1,',',b2,',',b3,',',c1,',',c2
   
!*** 3rd line (Water flux)
   write (Fn, '( f8.2, a, f8.2, a, f8.2, a, f8.2, a, f8.2 )') &
         hydRecord_preci,',', hydRecord_runoff,',', hydRecord_ic,',', hydRecord_ev,',', hydRecord_tr   
!*** 4th line ~ (for each PFT)
   !Sumup woody biomass (Mg C / ha)
   !  Reset variables
   c_leaf     (:) = 0.0
   c_trunk    (:) = 0.0
   c_root     (:) = 0.0
   c_stock    (:) = 0.0
   c_available(:) = 0.0
   
   !  Woody PFTs
   Do no=1, Max_no
   If (tree_exist(no)) then
      p = pft(no)
      c_leaf     (p) = c_leaf     (p) + mass_leaf(no)
      c_trunk    (p) = c_trunk    (p) + mass_trunk(no)
      c_root     (p) = c_root     (p) + mass_root(no)
      c_stock    (p) = c_stock    (p) + mass_stock(no)
      c_available(p) = c_available(p) + mass_available(no)
   End if
   End do
   
   x = C_in_drymass / Max_loc / Max_loc / 100.0
   c_leaf     (:) = c_leaf     (:) * x
   c_trunk    (:) = c_trunk    (:) * x
   c_root     (:) = c_root     (:) * x
   c_stock    (:) = c_stock    (:) * x
   c_available(:) = c_available(:) * x
   
   !Sunup grass biomass (Mg C / ha)
   x = C_in_drymass / 100.0
   if (type_grass ==3) then
      c_leaf     (C3g_no) = gmass_leaf     (1) * x * 0.9
      c_trunk    (C3g_no) = 0.0                         
      c_root     (C3g_no) = gmass_root     (1) * x * 0.9
      c_stock    (C3g_no) = gmass_stock    (1) * x * 0.9
      c_available(C3g_no) = gmass_available(1) * x * 0.9
      
      c_leaf     (C4g_no) = gmass_leaf     (2) * x * 0.1
      c_trunk    (C4g_no) = 0.0                         
      c_root     (C4g_no) = gmass_root     (2) * x * 0.1
      c_stock    (C4g_no) = gmass_stock    (2) * x * 0.1
      c_available(C4g_no) = gmass_available(2) * x * 0.1
   else
      c_leaf     (C3g_no) = gmass_leaf     (1) * x * 0.1
      c_trunk    (C3g_no) = 0.0                         
      c_root     (C3g_no) = gmass_root     (1) * x * 0.1
      c_stock    (C3g_no) = gmass_stock    (1) * x * 0.1
      c_available(C3g_no) = gmass_available(1) * x * 0.1
      
      c_leaf     (C4g_no) = gmass_leaf     (2) * x * 0.9
      c_trunk    (C4g_no) = 0.0                         
      c_root     (C4g_no) = gmass_root     (2) * x * 0.9
      c_stock    (C4g_no) = gmass_stock    (2) * x * 0.9
      c_available(C4g_no) = gmass_available(2) * x * 0.9
   endif
   
   DO p=1, PFT_no
      write (Fn,'( 7(f9.4,a) )') &
      gpp_month(p) * C_in_drymass / Max_loc / Max_loc / 100.0,',', &
      npp_month(p) * C_in_drymass / Max_loc / Max_loc / 100.0,',', &
      c_leaf(p),',',c_trunk(p),',',c_root(p),',',c_stock(p),',',c_available(p)
   END DO
   
END IF

!_____________ Write Annual output
IF ( Doy==365 ) then
   
   !Prepare size class distribution for each PFT
   size_dist(:,:) = 0.0 !reset
   DO no=1, Max_no
   If (tree_exist(no)) then
      
      p = pft(no)
      i = int(   (100*dbh_heartwood(no) + 100*dbh_sapwood(no)) / Dist_step   )
      
      if (i>=41) then
            size_dist(p,41) = size_dist(p,41) + 1
      elseif (i<1) then
            size_dist(p, 0) = size_dist(p, 0) + 1
      else
            size_dist(p, i) = size_dist(p, i) + 1
      end if
      
   End if
   END DO
   
   !Write size class ditribution
   DO p=1, PFT_no
   if ( (p.ne.c3g_no) .and. (p.ne.c4g_no) ) then
      do i = 0, 10
      write (Fn,'(i3,a$)') size_dist(p,i),','
      end do
      
      do i = 11, 20
      write (Fn,'(i2,a$)') size_dist(p,i),','
      end do
      
      do i = 21, 41
      write (Fn,'(i1,a$)') size_dist(p,i),','
      end do
      
      write (Fn,*)
   endif
   END DO
   
END IF

END SUBROUTINE output_for_viewer



!*************************************************************************************************
! output file maker (for global grid) (called @ the end of each month)
!*************************************************************************************************
SUBROUTINE output_global &
   (doy, counter, &
   tmp_air, tmp_soil, cloud, prec, humid, wind, &
   W_sat, &
   co2atm, ap, vp, vpd, dnsa, vps, slope_vps, &
   sl_hgt, dlen, rad, par, par_grass, &
   albedo_soil, albedo_veg, radnet_soil, radnet_veg, &
   mort_regu3, gdd, moist_factor, fire_factor, &
   pool_w1, pool_w2, pool_snow, canopy_cond, &
   pool_litter, pool_som_int, pool_som_slow, &
   water_status, eK, &
   pft, age, height, bole, diameter_h, diameter_s, crown_diameter, crown_area, la, &
   mass_f, mass_s, mass_r, mass_stock, mass_available, &
   tree_exist, height_limit, radius_limit,&
   type_grass, mass_f_grass, mass_r_grass, mass_available_grass, mass_stock_grass, &
   lai_grass, lai_opt_grass, &
   pft_exist, biome, &
   phenology, pheno_clo, pheno_cls, pheno_gdd, pheno_ncd, &
   c_emission, c_uptake, gpp_month, npp_month, gpp_annual, npp_annual, mort_regu1, &
   hydRecord_preci, hydRecord_runoff, hydRecord_ic, hydRecord_ev, hydRecord_tr)

USE data_structure
implicit none

!_____________ DEFINE AUGMENTS
!loop counters
integer,intent(IN)::doy       !day of the year (1-365)
integer,intent(IN)::counter   !simulation day (1-Simulation_year*365)

!Climate data
real,intent(IN)::tmp_air     !2m air temperature (Celcius)
real,intent(IN)::tmp_soil    !soil temperature (Celcius)
real,intent(IN)::cloud       !total cloudiness (fraction)
real,intent(IN)::prec        !precipitation (mm day-1)
real,intent(IN)::humid       !specific humidity (kg kg-1)
real,intent(IN)::wind        !wind  velocity (m s-1)

!Location data
real W_sat        !saturate point   (m3/m3, 0.0 -> 1.0)

!air related
real,intent(IN)::co2atm    !ambient (canopy) CO2 concentration (ppmv)
real,intent(IN)::ap        !air pressure (hPa)
real,intent(IN)::vp        !vapour pressure (hPa)
real,intent(IN)::vpd       !vapour pressure deficit (hPa)
real,intent(IN)::dnsa      !density of air (kg m-3)
real,intent(IN)::vps       !saturation vapour pressure (hPa)
real,intent(IN)::slope_vps !slope of saturation vapour pressure related to temp (hPa deg C-1)

!radiation related
real,intent(IN)::sl_hgt     !solar hight at midday (degree)
real,intent(IN)::dlen       !day length (hour)
real,intent(IN)::rad     !global radiation at the ground surface (W/m2)
real,intent(IN)::par        !par of mid-day (micro mol photon m-2 s-1)
real,intent(IN)::par_grass !par on top of the grass layer (micro mol photon m-2 s-1)

real,intent(IN)::albedo_soil  !soil surface albedo
real,intent(IN)::albedo_veg   !vegetation area albedo
real,intent(IN)::radnet_soil    !soil surface net radiation, short wave (W m-2)
real,intent(IN)::radnet_veg    !canopy net radiation, short wave (W m-2)

!climate related recorder
real,intent(IN)::mort_regu3   !heat stress for each year for mortality calc.
real,intent(IN)::gdd          !growing-degree-day-sum of 5 Celcius base
real,intent(IN)::moist_factor !adjusted moisture factor for fire spread
real,intent(IN)::fire_factor  !factor for fire probability calculation

!hydrogy related
real,intent(IN)::pool_w1      !soil water content of soil layer 1 (mm)
real,intent(IN)::pool_w2      !soil water content of soil layer 2 (mm)
real,intent(IN)::pool_snow    !water equivalent snow depth (mm)
real,intent(IN)::canopy_cond  !monthly stomatal conductance   (mol H2O m-2 s-1)

!soil carbon (s.o.m.; soil organic matter)
real,intent(IN)::pool_litter   !litter pool in a stand (g drymatter / stand)
real,intent(IN)::pool_som_int  !pool of s.o.m. with intermediated decomposition rate (g dm / stand)
real,intent(IN)::pool_som_slow !pool of s.o.m. with slow decomposition rate (g dm / stand)

!photosynthesis related
real,dimension(10)::water_status !state of water satisfactory (0.0-1.0)
real,dimension(10)::eK           !light attenuation coefficient, no dimension

!individual charactristics of woody PFT
integer,dimension(Max_no)::pft            !Plant Functional Type (1-10)
integer,dimension(Max_no)::age            !tree age (year)
integer,dimension(Max_no)::height         !tree height from 1.3m above ground (STEP)
integer,dimension(Max_no)::bole           !bole length from 1.3m above ground (STEP)
real   ,dimension(Max_no)::diameter_h     !heartwood diameter at 1.3m   (m)
real   ,dimension(Max_no)::diameter_s     !sapwood   diameter at 1.3m   (m)
real   ,dimension(Max_no)::crown_diameter !crown diameter (m)
real   ,dimension(Max_no)::crown_area     !crown area     (m2)
real   ,dimension(Max_no)::la             !leaf area      (m2)
real   ,dimension(Max_no)::mass_f         !biomass of foliage (g)
real   ,dimension(Max_no)::mass_s         !biomass of stem    (g)
real   ,dimension(Max_no)::mass_r         !biomass of root    (g)
real   ,dimension(Max_no)::mass_stock     !biomass for stock  (g)
real   ,dimension(Max_no)::mass_available !biomass available  (g)
logical,dimension(Max_no)::tree_exist     !tree exist flag
integer,dimension(Max_no)::height_limit   !maximum tree height that proximate trees permit
real   ,dimension(Max_no)::radius_limit   !maximum canopy radius that proximate trees permit

!grid charactristics of herbaceous PFT
integer type_grass           !grass type priority in this girid (3:C3 type, 4:C4 type)

real,dimension(2)::mass_f_grass         !c3/c4 herbaceous foliage   biomass in a unit area (g/m2)
real,dimension(2)::mass_r_grass         !c3/c4 herbaceous root      biomass in a unit area (g/m2)
real,dimension(2)::mass_available_grass !c3/c4 herbaceous available biomass in a unit area (g/m2)
real,dimension(2)::mass_stock_grass     !c3/c4 herbaceous stock     biomass in a unit area (g/m2)
real,dimension(2)::lai_grass            !c3/c4 leaf area index of grass (m2/m2)
real,dimension(2)::lai_opt_grass        !c3/c4 optimum LAI for grass (m2/m2)

!biome stat
logical,dimension(10)::pft_exist !flag: PFT exist or not (.TRUE. or .FALSE.)
integer                ::biome     !biome type number (see subroutine biome_type for definition)

!phenology related
logical,dimension(10)::phenology !Phenological status (T:foliation, F:dormance)
integer,dimension(10)::pheno_clo !counter number of last leaf onset
integer,dimension(10)::pheno_cls !counter number of last leaf shedding
real                   ::pheno_gdd !GDD on 5 degree base
                                   !(reset on 1 Jan for N-hemisphere, 1 July for S-hemisphere)
                                   !
integer                ::pheno_ncd !Number of chilling day, defined as number of days on which
                                   !the daily mean air temperature is below 5 Cecius degree
                                   !(reset on 1 Nov for N-hemisphere, 1 May for S-hemisphere)

!carbon flux recorder
real                  c_emission !monthly carbon emission (C g / month stand)
real                  c_uptake   !monthly carbon uptake   (C g / month stand)
real,dimension(10)::gpp_month  !GPP for each pft (g dm / stand / month)
real,dimension(10)::npp_month  !NPP for each pft (g dm / stand / month)
real,dimension(10)::gpp_annual !GPP for each pft (g dm / stand / year)
real,dimension(10)::npp_annual !NPP for each pft (g dm / stand / year)
real,dimension(Max_no)::mort_regu1 !NPP annual (g dm / individual)

!hydrogy record
real hydRecord_preci  !sum of precipitation      (mm/month)
real hydRecord_runoff !sum of runoff water       (mm/month)
real hydRecord_ic     !sum of interception water (mm/month)
real hydRecord_ev     !sum of evaporated water   (mm/month)
real hydRecord_tr     !sum of tranpirated water  (mm/month)

!_____________ DEFINE LOCAL VARIABLES
!for output variables
real woody_carbon, grass_carbon, som1, som2, som3 !(kg C / m2)
real gpp_out, npp_out, nep_out                    !(kg C / m2 / month)
real mean_woody_mass      !(g C / tree)
real tree_density         !(N / ha)
real woody_lai, grass_lai !(m2/m2)

integer  Fn, no, i
real     x

!_____________ Set I/O number
Fn = File_no(20)

!_____________ Prepare output variables
   !common procedure
   x = 0.0 !sum of tree biomass (g dm)
   i = 0   !sum of tree number        
   do no=1, Max_no
   if (tree_exist(no)) then
      x = x + mass_f(no)+mass_s(no)+mass_r(no)+mass_stock(no)+mass_available(no)
      i = i + 1
   endif
   enddo
   
   !tree_density (N / ha)
   tree_density = 10000.0 * real(i) / Max_loc / Max_loc
   
   !mean_woody_mass (kg C / tree)
   mean_woody_mass = x * C_in_drymass / max(1.0, real(i)) / 1000.0
   
   !woody_carbon (kg C / m2)
   woody_carbon = x * C_in_drymass / Max_loc / Max_loc / 1000.0
   
   !grass_carbon (kg C / m2)
   if (type_grass==3) then
     grass_carbon = &
     0.9*(mass_f_grass(1)+mass_r_grass(1)+mass_stock_grass(1)+mass_available_grass(1))+ &
     0.1*(mass_f_grass(2)+mass_r_grass(2)+mass_stock_grass(2)+mass_available_grass(2))
   else
     grass_carbon = &
     0.1*(mass_f_grass(1)+mass_r_grass(1)+mass_stock_grass(1)+mass_available_grass(1))+ &
     0.9*(mass_f_grass(2)+mass_r_grass(2)+mass_stock_grass(2)+mass_available_grass(2))
   endif
   grass_carbon = grass_carbon * C_in_drymass / 1000.0
   
   !soil_carbon (kg C / m2)
   som1 = pool_litter   * C_in_drymass / Max_loc / Max_loc / 1000.0
   som2 = pool_som_int  * C_in_drymass / Max_loc / Max_loc / 1000.0
   som3 = pool_som_slow * C_in_drymass / Max_loc / Max_loc / 1000.0
   
   !GPP (kg C / m2 / month)
   gpp_out = sum(gpp_month) * C_in_drymass / Max_loc / Max_loc / 1000.0
   
   !NPP (kg C / m2 / month)
   npp_out = sum(npp_month) * C_in_drymass / Max_loc / Max_loc / 1000.0
   
   !NEP (kg C / m2 / month)
   nep_out = (c_uptake-c_emission) / Max_loc / Max_loc / 1000.0
   
   !Tree LAI (m2/m2)
   woody_lai = sum(la) / Max_loc / Max_loc
   
   !Grass LAI (m2/m2)
   if (type_grass==3) then
     grass_lai = 0.9 * lai_grass(1) + 0.1 * lai_grass(2)
   else
     grass_lai = 0.1 * lai_grass(1) + 0.9 * lai_grass(2)
   endif
   
!_____________ Write output-data
write (Fn,'( i2, 5(1x,f7.2), 3(1x,f9.1), 3(1x,f8.3), 1(1x,f6.1), 2(1x,f4.1), 4(1x,f6.1), 1x, f6.0, 1x, f6.2 )') &
   biome            , & !1: Biome no (classfication)
   
   woody_carbon     , & !2: Carbon in Woody biomass [kg C / m2]
   grass_carbon     , & !3: Carbon in Grass biomass [kg C / m2]
   som1             , & !4: Carbon in litter        [kg C / m2]
   som2             , & !5: Carbon in som_int       [kg C / m2]
   som3             , & !6: Carbon in som_slow      [kg C / m2]
   
   pool_w1          , & !7: Water in soil layer 1 [mm]
   pool_w2          , & !8: Water in soil layer 2 [mm]
   pool_snow        , & !9: Water in snow         [mm]
   
   gpp_out          , & !10: GPP [kg C / m2 / month]
   npp_out          , & !11: NPP [kg C / m2 / month]
   nep_out          , & !12: NEP [kg C / m2 / month]
   
   mean_woody_mass  , & !13: mean_woody_mass   [kg C / tree]
   woody_lai        , & !14: LAI of woody PFTs [m2/m2]
   grass_lai        , & !15: LAI of grass PFTs [m2/m2]
   
   hydRecord_runoff , & !16: runoff        [mm/month]
   hydRecord_ic     , & !17: interception  [mm/month]
   hydRecord_ev     , & !18: evaporation   [mm/month]
   hydRecord_tr     , & !19: transpiration [mm/month]
   
   tree_density     , & !20: tree_density  [N / ha]
   
   canopy_cond          !21: monthly stomatal conductance [mol H2O m-2 s-1]

END SUBROUTINE output_global
